@@ -9,24 +9,26 @@
 #' @param projectId The projectId where it all gets deployed to
 #' @param region The Cloud Run endpoint set by CR_REGION env arg
 #' @param bucket The Cloud Storage bucket that will hold the code
+#' @param pre_steps Other \link{cr_buildstep} to run before the docker build
+#' @param post_steps Other \link{cr_buildstep} to run after the docker build
 #' @param ... Other arguments passed to \link{cr_buildstep_run}
 #' @inheritDotParams cr_buildstep_run
 #' @inheritParams cr_buildstep_docker
 #' @inheritParams cr_build
 #' @family Deployment functions
 #' @details
-#'
 #' These deploy containers to Cloud Run, a scale 0-to-millions container-as-a-service on Google Cloud Platform.
 #'
+#' @seealso For scheduling Cloud Run apps \link{cr_run_schedule_http}
+#'
+#'
 #' @export
 #' @examples
-#'
 #' \dontrun{
 #' cr_project_set("my-project")
 #' cr_region_set("europe-west1")
 #' cr_bucket_set("my-bucket")
 #' cr_deploy_run(system.file("example/", package = "googleCloudRunner"))
-#'
 #' }
 cr_deploy_run <- function(local,
                           remote = basename(local),
@@ -37,9 +39,11 @@ cr_deploy_run <- function(local,
                           bucket = cr_bucket_get(),
                           projectId = cr_project_get(),
                           launch_browser = interactive(),
-                          timeout=600L,
-                          ...){
-
+                          timeout = 600L,
+                          kaniko_cache = TRUE,
+                          pre_steps = NULL,
+                          post_steps = NULL,
+                          ...) {
   assert_that(
     is.dir(local),
     is.string(remote),
@@ -51,37 +55,44 @@ cr_deploy_run <- function(local,
   image_name <- make_image_name(image_name, projectId)
 
   built <- cr_deploy_docker(local,
-                            image_name = image_name,
-                            dockerfile = dockerfile,
-                            remote = remote,
-                            tag = tag,
-                            bucket = bucket,
-                            projectId = projectId,
-                            launch_browser = launch_browser,
-                            timeout=timeout,
-                            kaniko_cache=TRUE)
+    image_name = image_name,
+    dockerfile = dockerfile,
+    remote = remote,
+    tag = tag,
+    bucket = bucket,
+    projectId = projectId,
+    launch_browser = launch_browser,
+    timeout = timeout,
+    kaniko_cache = kaniko_cache,
+    pre_steps = pre_steps,
+    post_steps = post_steps
+  )
 
-  if(built$status != "SUCCESS"){
+  if (built$status != "SUCCESS") {
     myMessage("Error building Dockerfile", level = 3)
     return(built)
   }
 
-  built_image <- built$results$images$name
+  built_image <- paste0(image_name, ":", built$id)
+
+  if (is.null(built_image)) {
+    stop("Could not find image name in built response", call. = FALSE)
+  }
   myMessage("Built Docker image: ", built_image, level = 3)
 
   cr_run(built_image,
-         name = lower_alpha_dash(remote),
-         region = region,
-         projectId = projectId,
-         launch_browser=launch_browser,
-         timeout=timeout,
-         ...)
-
+    name = lower_alpha_dash(remote),
+    region = region,
+    projectId = projectId,
+    launch_browser = launch_browser,
+    timeout = timeout,
+    ...
+  )
 }
 
 #' @param html_folder the folder containing all the html
 #' @inheritParams cr_deploy_run
-#' @details
+#' @param ... Other arguments passed to \link{cr_deploy_run} and eventually \link{cr_buildstep_run}
 #'
 #' @section cr_deploy_html:
 #' Deploy html files to a nginx server on Cloud Run.
@@ -95,14 +106,12 @@ cr_deploy_run <- function(local,
 #' @export
 #' @import assertthat
 #' @examples
-#'
 #' \dontrun{
 #' cr_project_set("my-project")
 #' cr_region_set("europe-west1")
 #' cr_bucket_set("my-bucket")
 #'
 #' cr_deploy_html("my_folder")
-#'
 #' }
 #' @rdname cr_deploy_run
 cr_deploy_html <- function(html_folder,
@@ -113,31 +122,38 @@ cr_deploy_html <- function(html_folder,
                            bucket = cr_bucket_get(),
                            projectId = cr_project_get(),
                            launch_browser = interactive(),
-                           timeout=600L){
+                           timeout = 600L,
+                           ...) {
+  file.copy(
+    from = system.file("docker", "nginx", "default.template",
+      package = "googleCloudRunner"
+    ),
+    to = file.path(html_folder, "default.template"),
+    overwrite = TRUE
+  )
 
-  file.copy(from = system.file("docker","nginx","default.template",
-                               package = "googleCloudRunner"),
-            to = file.path(html_folder, "default.template"),
-            overwrite = TRUE)
-
-  cr_deploy_run(local = html_folder,
-                remote = remote,
-                dockerfile = system.file("docker","nginx","Dockerfile",
-                                         package = "googleCloudRunner"),
-                image_name = image_name,
-                tag = tag,
-                region =  region,
-                bucket = bucket,
-                projectId = projectId,
-                launch_browser = launch_browser,
-                timeout=timeout)
+  cr_deploy_run(
+    local = html_folder,
+    remote = remote,
+    dockerfile = system.file("docker", "nginx", "Dockerfile",
+      package = "googleCloudRunner"
+    ),
+    image_name = image_name,
+    tag = tag,
+    region = region,
+    bucket = bucket,
+    projectId = projectId,
+    launch_browser = launch_browser,
+    timeout = timeout,
+    ...
+  )
 }
 
 #' Deploy a plumber API
 #' @rdname cr_deploy_run
 #' @param api A folder containing the R script using plumber called api.R and all its dependencies
+#' @param ... Other arguments passed to \link{cr_deploy_run} and eventually \link{cr_buildstep_run}
 #'
-#' @details
 #'
 #' @section cr_deploy_plumber:
 #'
@@ -148,14 +164,12 @@ cr_deploy_html <- function(html_folder,
 #'
 #' @export
 #' @examples
-#'
 #' \dontrun{
 #' cr_project_set("my-project")
 #' cr_region_set("europe-west1")
 #' cr_bucket_set("my-bucket")
 #'
 #' cr_deploy_plumber(system.file("example/", package = "googleCloudRunner"))
-#'
 #' }
 cr_deploy_plumber <- function(api,
                               remote = basename(api),
@@ -166,36 +180,27 @@ cr_deploy_plumber <- function(api,
                               bucket = cr_bucket_get(),
                               projectId = cr_project_get(),
                               launch_browser = interactive(),
-                              timeout=600L){
-
+                              timeout = 600L,
+                              ...) {
   local <- api
   local_files <- list.files(local)
-  if(!"api.R" %in% local_files){
+  if (!"api.R" %in% local_files) {
     stop("Must include api.R in local deployment folder
          with library(plumber) implementation
          for Cloud Run deployments", call. = FALSE)
   }
 
-  # if no dockerfile, attempt to create it
-  if(is.null(dockerfile)){
-    if(!"Dockerfile" %in% local_files){
-      myMessage("No Dockerfile detected in ",local, level = 3)
-      cr_dockerfile_plumber(local)
-    } else {
-      myMessage("Using existing Dockerfile found in folder", level = 3)
-    }
-  }
-
-  cr_deploy_run(local = local,
-                remote = remote,
-                dockerfile = dockerfile,
-                image_name = image_name,
-                tag = tag,
-                region =  region,
-                bucket = bucket,
-                projectId = projectId,
-                launch_browser = launch_browser,
-                timeout=timeout)
-
+  cr_deploy_run(
+    local = local,
+    remote = remote,
+    dockerfile = dockerfile,
+    image_name = image_name,
+    tag = tag,
+    region = region,
+    bucket = bucket,
+    projectId = projectId,
+    launch_browser = launch_browser,
+    timeout = timeout,
+    ...
+  )
 }
-
